PanicHookInfo

Struct PanicHookInfo 

1.81.0 · Source
pub struct PanicHookInfo<'a> {
    payload: &'a (dyn Any + Send),
    location: &'static Location<'static>,
    can_unwind: bool,
    force_no_backtrace: bool,
}
A struct providing information about a panic.

PanicHookInfo structure is passed to a panic hook set by the set_hook function.

Examples

use std::panic;

panic::set_hook(Box::new(|panic_info| {
    println!("panic occurred: {panic_info}");
}));

panic!("critical system failure");

Fields

payload: &'a (dyn Any + Send)location: &'static Location<'static>can_unwind: boolforce_no_backtrace: bool

Implementations

Source

impl<'a> PanicHookInfo<'a>

pub(crate) fn new( location: &'static Location<'static>, payload: &'a (dyn Any + Send), can_unwind: bool, force_no_backtrace: bool, ) -> Self

1.10.0 · Source

pub fn payload(&self) -> &(dyn Any + Send)

Returns the payload associated with the panic.

This will commonly, but not always, be a &'static str or String. If you only care about such payloads, use payload_as_str instead.

A invocation of the panic!() macro in Rust 2021 or later will always result in a panic payload of type &'static str or String.

Only an invocation of panic_any (or, in Rust 2018 and earlier, panic!(x) where x is something other than a string) can result in a panic payload other than a &'static str or String.

Examples
use std::panic;

panic::set_hook(Box::new(|panic_info| {
    if let Some(s) = panic_info.payload().downcast_ref::<&str>() {
        println!("panic occurred: {s:?}");
    } else if let Some(s) = panic_info.payload().downcast_ref::<String>() {
        println!("panic occurred: {s:?}");
    } else {
        println!("panic occurred");
    }
}));

panic!("Normal panic");
1.91.0 · Source

pub fn payload_as_str(&self) -> Option<&str>

Returns the payload associated with the panic, if it is a string.

This returns the payload if it is of type &'static str or String.

A invocation of the panic!() macro in Rust 2021 or later will always result in a panic payload where payload_as_str returns Some.

Only an invocation of panic_any (or, in Rust 2018 and earlier, panic!(x) where x is something other than a string) can result in a panic payload where payload_as_str returns None.

Example
std::panic::set_hook(Box::new(|panic_info| {
    if let Some(s) = panic_info.payload_as_str() {
        println!("panic occurred: {s:?}");
    } else {
        println!("panic occurred");
    }
}));

panic!("Normal panic");
1.10.0 · Source

pub fn location(&self) -> Option<&'static Location<'static>>

Returns information about the location from which the panic originated, if available.

This method will currently always return Some, but this may change in future versions.

Examples
use std::panic;

panic::set_hook(Box::new(|panic_info| {
    if let Some(location) = panic_info.location() {
        println!("panic occurred in file '{}' at line {}",
            location.file(),
            location.line(),
        );
    } else {
        println!("panic occurred but can't get location information...");
    }
}));

panic!("Normal panic");
pub fn can_unwind(&self) -> bool

🔬This is a nightly-only experimental API. (panic_can_unwind #92988)

Returns whether the panic handler is allowed to unwind the stack from the point where the panic occurred.

This is true for most kinds of panics with the exception of panics caused by trying to unwind out of a Drop implementation or a function whose ABI does not support unwinding.

It is safe for a panic handler to unwind even when this function returns false, however this will simply cause the panic handler to be called again.

#[doc(hidden)]
pub fn force_no_backtrace(&self) -> bool

🔬This is a nightly-only experimental API. (panic_internals)